#ifndef SHARED_COMMON_H
#define SHARED_COMMON_H
//
uniform float3x4	m_W;
uniform float3x4	m_V;
uniform float4x4 	m_P;
uniform float3x4	m_WV;
uniform float4x4 	m_VP;
uniform float4x4 	m_WVP;
uniform float4		timers;
uniform float4		fog_plane;
uniform float4		fog_params;		// x=near*(1/(far-near)), ?,?, w = -1/(far-near)
uniform float4		fog_color;
uniform float3		L_sun_color;
uniform float3		L_sun_dir_w;
uniform float3		L_sun_dir_e;
uniform float4		L_hemi_color;
uniform float4		L_ambient;		// L_ambient.w = skynbox-lerp-factor
uniform float3 		eye_position;
uniform float3		eye_direction;
uniform float3		eye_normal;
uniform	float4 		dt_params;

// new uniform variables - OGSE Team
// global constants
uniform float4 ogse_c_resolution; // x - width, y - height, z - 1/width, w - 1/height
uniform float4 ogse_c_screen; // x - fFOV, y - fAspect, z - Zf/(Zf-Zn), w - Zn*tan(fFov/2)
uniform float4 ogse_c_jitter; // x - jitter u, y - jitter v. Test for smaa t2x, don't touch
uniform float4 ogse_c_rain; // x - rmap dist, y - flood level, z - wet level, w - rain intensity


#define PI  3.14159265f
#define float_PI 1.570796325f
#define DOUBLE_PI  6.2831853f

float3 	unpack_normal	(float3 v)	{ return 2*v-1;			}
float3 	unpack_bx2	(float3 v)	{ return 2*v-1; 		}
float3 	unpack_bx4	(float3 v)	{ return 4*v-2; 		}

float2 	unpack_tc_base	(float2 tc, float du, float dv)		{
		return (tc.xy + float2	(du,dv))*(32.f/32768.f);
}

float2 	unpack_tc_lmap	(float2 tc)	{ return tc*(1.f/32768.f);	} // [-1  .. +1 ]

float 	calc_cyclic 	(float x)				{
	float 	phase 	= 1/(2*3.141592653589f);
	float 	sqrt2	= 1.4142136f;
	float 	sqrt2m2	= 2.8284271f;
	float 	f 	= sqrt2m2*frac(x)-sqrt2;	// [-sqrt2 .. +sqrt2]
	return 	f*f - 1.f;				// [-1     .. +1]
}
float2 	calc_xz_wave 	(float2 dir2D, float frac)		{
	// Beizer
	float2  ctrl_A	= float2(0.f,		0.f	);
	float2 	ctrl_B	= float2(dir2D.x,	dir2D.y	);
	return  lerp	(ctrl_A, ctrl_B, frac);
}

float4 proj_to_screen(float4 proj)
{
	float4 screen = proj;
	screen.x = (proj.x + proj.w);
	screen.y = (proj.w - proj.y);
	screen.xy *= 0.5;
	return screen;
}
float4 screen_to_proj(float2 screen, float z)
{
	float4 proj;
	proj.w = 1.0;
	proj.z = z;
	proj.x = screen.x*2 - proj.w;
	proj.y = -screen.y*2 + proj.w;
	return proj;
}
float is_in_range(float3 args)
{
	float mn = (args.x > args.y) ? 1: 0;
	float mx = (args.z > args.x) ? 1: 0;
	return mn*mx;
}

/*
float _sin(float angle)
{
	float _angle = (angle > PI)?(angle - PI):angle;
	_angle = (_angle > float_PI)?(PI - _angle):_angle;
	float mult = _angle*_angle*_angle/6;
	return (_angle-mult);
}
float c_sin(float angle)
{
	float sig = (angle > PI)?(-1):1;
	return sig*_sin(angle);
}
float mod (float x, float y)
{
	return (x-y*floor(x/y));
}
float random( float2 p , float scale)
{
	float2 tc = ceil(p*1024/scale)*scale/1024;
  // We need irrationals for pseudo randomness.
  // Most (all?) known transcendental numbers will (generally) work.
  const float2 r = float2(
    23.1406926327792690,  // e^pi (Gelfond's constant)
     2.6651441426902251); // 2^sqrt(2) (GelfondSchneider constant)
  return frac( cos( mod( 123456789., 1e-7 + 256. * dot(tc,r) ) ) );  
}*/
#endif
